# coding=utf-8
import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as mpt
import matplotlib as mpl
from matplotlib import rc

##
def bandPlot(ax):
	firstPoint = 0
	lastPoint = kpathPoints
	while lastPoint<=len(data_bands):
		kPlot = kpath[firstPoint:lastPoint]
		bandPlot = band[firstPoint:lastPoint]
		line1, = ax.plot(kPlot,bandPlot, color='grey', linestyle='-', linewidth='3.5',  alpha = 0.3, antialiased=True, zorder=5, label='5\,GPa')
		firstPoint = lastPoint
		lastPoint = lastPoint + kpathPoints
    
	firstPoint = 0
	lastPoint = kpathPoints
	while lastPoint<=len(data_bands_0):
		kPlot = kpath_0[firstPoint:lastPoint]
		bandPlot = band_0[firstPoint:lastPoint]
		line2, = ax.plot(kPlot,bandPlot, color='blue', linestyle='-', linewidth='3.5',  alpha = 0.3, antialiased=True, zorder=5, label='0\,GPa')
		firstPoint = lastPoint
		lastPoint = lastPoint + kpathPoints

	ax.legend(handles=[line2,line1], loc='best', fontsize='12')
##

## setting physical parameters
kpathPoints=901 ## number of k-points along the high-symmetry path within first Brillouin zone
kpathCoordinate = 6.0656 ## last point in the high-symmetry path
fermi_energy_bands = 17.6971 ## Fermi energy in the scf output file
fermi_energy_dos = 17.7383 ## Fermi energy in the nscf output file
fermi_energy_0 = 16.5153

## path to the electronic band structure file
file_bands = './bands.dat.gnu'
file_bands_0 = './bands_0GPa.dat.gnu'

## band structure data import from file
data_bands = np.genfromtxt(file_bands, usemask='True')
data_bands_0 = np.genfromtxt(file_bands_0, usemask='True')

## extracting the data as numpy arrays
kpath = data_bands[:,0] ## high-symmetry path
band = data_bands[:,1] - fermi_energy_bands ## electronic bands
kpath_0 = data_bands_0[:,0] ## high-symmetry path
band_0 = data_bands_0[:,1] - fermi_energy_0 ## electronic bands

## path to the electronic density of states file
file_dos_tot = './LuH.dos' 
file_dos_Lu_s = './pdos_Lu_s.dat' 
file_dos_Lu_p = './pdos_Lu_p.dat' 
file_dos_Lu_d = './pdos_Lu_d.dat' 
file_dos_Lu_f = './pdos_Lu_f.dat'
file_dos_H_s = './pdos_H_s.dat'

## density of states data import from file
data_dos_tot = np.genfromtxt(file_dos_tot, usemask='True')
data_dos_Lu_s = np.genfromtxt(file_dos_Lu_s, usemask='True')
data_dos_Lu_p = np.genfromtxt(file_dos_Lu_p, usemask='True')
data_dos_Lu_d = np.genfromtxt(file_dos_Lu_d, usemask='True')
data_dos_Lu_f = np.genfromtxt(file_dos_Lu_f, usemask='True')
data_dos_H_s = np.genfromtxt(file_dos_H_s, usemask='True')

## extracting the data as numpy arrays
energy_tot = data_dos_tot[:,0] - fermi_energy_dos ## energy for the PDOS plot
dos_tot = data_dos_tot[:,1] ## total density of states
energy = data_dos_Lu_s[:,0] - fermi_energy_dos
dos_Lu_s = data_dos_Lu_s[:,1]
dos_Lu_p = data_dos_Lu_p[:,1]
dos_Lu_d = data_dos_Lu_d[:,1]
dos_Lu_f = data_dos_Lu_f[:,1]
dos_H_s = data_dos_H_s[:,1]

## plotting

## defining the LaTeX font style
rc('font',**{'family':'Palatino','serif':['Palatino'],'size':16})
rc('text', usetex=True)

## defining the axes and line widths
mpl.rcParams['axes.linewidth'] = 3
mpl.rcParams['xtick.major.width'] = 3
mpl.rcParams['ytick.major.width'] = 3

## setting the subplot framework
fig, ax = plt.subplots(1, 2, figsize=(7.5,5), sharey=True, gridspec_kw={'width_ratios': [3, 1]})
plt.subplots_adjust(wspace=0.15)

## plotting phonon band structure

ax1 = ax[0]

## k-path coordinates
X =  1.0000
W =  1.5000
K =  1.8536
G =  2.9142
L =  3.7802
U  =  4.3926
W =  4.7462
L =  5.4533

## drawing the high-symmetry path labels
xlabels = {0:'$\Gamma$', X:'X', W:'W', K:'K', G:'G', L:'L', U:'U', W:'W', L:'L', kpathCoordinate:'K'}
xmajorLocator = mpt.FixedLocator([x for x in xlabels])
xmajorFormat = mpt.FixedFormatter([x for x in xlabels.values()])
ax1.xaxis.set_major_locator(xmajorLocator)
ax1.xaxis.set_major_formatter(xmajorFormat)

## defining the axes and labels for the phonon band structure plot
ax1.tick_params(axis='both',direction='in',bottom=True, top=True, left=True, right=True)
ax1.set_xlim(0, kpathCoordinate)
ax1.set_ylim(-4,4)
ax1.set_ylabel('Energy\,(eV)')

## plotting the phonon band structure
ax1.axhline(y=0, color='black', linestyle='--', linewidth='3', zorder=1, alpha=0.8)

## drawing the high-symmetry lines vertically
ax1.axvline(x=X, color='black', linewidth='3.5')
ax1.axvline(x=W, color='black', linewidth='3.5')
ax1.axvline(x=K, color='black', linewidth='3.5')
ax1.axvline(x=G, color='black', linewidth='3.5')
ax1.axvline(x=L, color='black', linewidth='3.5')
ax1.axvline(x=U, color='black', linewidth='3.5')
ax1.axvline(x=W, color='black', linewidth='3.5')
ax1.axvline(x=L, color='black', linewidth='3.5')
bandPlot(ax1)

## plotting the density of states
ax2 = ax[1]
ax2.tick_params(axis='both',direction='in',bottom=True, top=True, left=True, right=True)
ax2.set_xlim(0,2)
ax2.set_xticks((np.arange(0,3,step=1)))
ax2.axhline(y=0, color='black', linestyle='--', linewidth='3.5')
ax2.set_xlabel('DOS\,(states/eV)')
plt.setp(ax2.get_yticklabels(), visible=False)
line1, = ax2.plot(dos_tot, energy_tot, color='black', linestyle='-', linewidth='3.5', antialiased=True, alpha = 0.9, label='Total')
ax2.fill_betweenx(energy_tot, dos_tot , color='black', alpha=0.1)
line2, = ax2.plot(dos_Lu_d, energy, color='green', linestyle='-', linewidth='3.5', antialiased=True, alpha = 0.6, label='Lu-d')
ax2.fill_betweenx(energy, dos_Lu_d, color='green', alpha=0.15)
line3, = ax2.plot(dos_H_s, energy, color='red', linestyle='-', linewidth='3.5', antialiased=True, alpha = 0.6, label='H-s')
ax2.fill_betweenx(energy, dos_H_s, color='red', alpha=0.15)
ax2.legend(handles=[line1,line2,line3], loc=1, fontsize='8')

## saving the figures
plt.savefig('bands.png', bbox_inches='tight', dpi=300)