# coding=utf-8

import numpy as np
import matplotlib.pyplot as plt
import matplotlib.ticker as mpt
import matplotlib as mpl
import scipy.integrate as integrate
from matplotlib import rc
##
def bandPlot(ax):
	firstPoint = 0
	lastPoint = kpathPoints
	while lastPoint<=len(data):
		kPlot = kpath[firstPoint:lastPoint]
		bandPlot = band[firstPoint:lastPoint]
		line1, = ax.plot(kPlot,bandPlot, color='blue', linestyle='-', linewidth='3.5',  alpha = 0.4, antialiased=True, label='0\,GPa')
		firstPoint = lastPoint
		lastPoint = lastPoint + kpathPoints
	firstPoint = 0
	lastPoint = kpathPoints
	while lastPoint<=len(data_5):
		kPlot = kpath_5[firstPoint:lastPoint]
		bandPlot = band_5[firstPoint:lastPoint]
		line2, = ax.plot(kPlot,bandPlot, color='grey', linestyle='-', linewidth='3.5',  alpha = 0.3, antialiased=True, label='5\,GPa')
		firstPoint = lastPoint
		lastPoint = lastPoint + kpathPoints
	ax.legend(handles=[line1,line2], loc='best', fontsize='12')
##

## setting physical parameters
kpathPoints=701 ## number of k-points along the high-symmetry path within first Brillouin zone
kpathCoordinate = 3.4335 ## last point in the high-symmetry path
cm_to_meV = 1/8.066 ## converting cm-1 to meV 
THz_to_meV = 4.136 ## converting THz to meV
degauss = 7 ## n-th column in the alpha2F.dat file (converged smearing)

## path to the phonon band structure file
file = './phonons.dat.gnu'
file_5 = './phonons_5GPa.dat.gnu'

## phonon band structure data import from file
data = np.genfromtxt(file, usemask='True')
data_5 = np.genfromtxt(file_5, usemask='True')

## extracting the data as numpy arrays
kpath = data[:,0] ## high-symmetry path
band = data[:,1]*cm_to_meV ## phonon bands
kpath_5 = data_5[:,0] ## high-symmetry path
band_5 = data_5[:,1]*cm_to_meV ## phonon bands

## path to the phonon density of states file
file_dos = './phonon.dos' 

## phonon density of states data import from file
data_dos = np.genfromtxt(file_dos, usemask='True')

## extracting the data as numpy arrays
freq = data_dos[:,0]*cm_to_meV ## frequency for the PDOS plot
dos_tot = data_dos[:,1] ## total density of states
dos_H = data_dos[:,4]+data_dos[:,5]+data_dos[:,6]+data_dos[:,7]+data_dos[:,8]+data_dos[:,9]
dos_Lu = data_dos[:,2]+data_dos[:,3]

## path to the Eliashberg function file
file_a2f = './alpha2F.dat'

## Eliashberg function data data import from file
data_a2f = np.genfromtxt(file_a2f, usemask='True')

## extracting the data as numpy arrays
freq_a2f = data_a2f[:,0]*THz_to_meV ## frequency for the Eliashberg function plot
a2f = data_a2f[:,degauss] ## Eliashberg function

## calculating the electron-phonon coupling from the Eliashberg function
elph = np.zeros(len(freq_a2f)) ## creating the numpy array for the electron-phonon coupling
for i in range(len(elph)): 
    elph[i] = 2*integrate.simpson(a2f[:i+1]/freq_a2f[:i+1],freq_a2f[:i+1]) ## integrating the acumulative electron-phonon coupling 

## plotting

## defining the LaTeX font style
rc('font',**{'family':'Palatino','serif':['Palatino'],'size':16})
rc('text', usetex=True)

## defining the axes and line widths
mpl.rcParams['axes.linewidth'] = 3
mpl.rcParams['xtick.major.width'] = 3
mpl.rcParams['ytick.major.width'] = 3

## setting the subplot framework
fig, ax = plt.subplots(1, 3, figsize=(10,5), sharey=True, gridspec_kw={'width_ratios': [3, 1, 1]})
plt.subplots_adjust(wspace=0.15)

## plotting the phonon band structure
ax1 = ax[0]

## k-path coordinates
M =  0.5774
K =  0.9107
G =  1.5774
A =  1.8561
L =  2.4335
H  =  2.7668

## drawing the high-symmetry path labels
xlabels = {0:'$\Gamma$', M:'M', K:'K', G:'$\Gamma$', A:'A', L:'L', H:'H', kpathCoordinate:'A'}
xmajorLocator = mpt.FixedLocator([x for x in xlabels])
xmajorFormat = mpt.FixedFormatter([x for x in xlabels.values()])
ax1.xaxis.set_major_locator(xmajorLocator)
ax1.xaxis.set_major_formatter(xmajorFormat)

## defining the axes and labels for the phonon band structure plot
ax1.tick_params(axis='both',direction='in',bottom=True, top=True, left=True, right=True)
ax1.set_xlim(0, kpathCoordinate)
ax1.set_ylim(-80,200)
ax1.set_ylabel('$\hbar\omega$\,(meV)')

## plotting the phonon band structure
bandPlot(ax1)

## drawing the high-symmetry lines vertically
ax1.axvline(x=M, color='black', linewidth='3', zorder=1, alpha=0.8)
ax1.axvline(x=K, color='black', linewidth='3', zorder=1, alpha=0.8)
ax1.axvline(x=G, color='black', linewidth='3', zorder=1, alpha=0.8)
ax1.axvline(x=A, color='black', linewidth='3', zorder=1, alpha=0.8)
ax1.axvline(x=L, color='black', linewidth='3', zorder=1, alpha=0.8)
ax1.axvline(x=H, color='black', linewidth='3', zorder=1, alpha=0.8)
ax1.axhline(y=0, color='black', linestyle='--', linewidth=3)

## plotting the phonon density of states
ax2 = ax[1]
ax2.tick_params(axis='both',direction='in',bottom=True, top=True, left=True, right=True)
ax2.set_xlim(0,0.05)
ax2.set_xticks((np.arange(0,0.06,step=0.02)))
ax2.set_xlabel('PDOS\,(meV$^{-1}$)')
plt.setp(ax2.get_yticklabels(), visible=False)
ax2.axhline(y=0, color='black', linestyle='--', linewidth=3)
line1, = ax2.plot(dos_tot, freq,  color='black', linestyle='-', linewidth='3.5', antialiased=True, alpha = 0.9, label='Total')
ax2.fill_between(dos_tot, freq ,color='black', alpha=0.15)
line2, = ax2.plot(dos_Lu, freq, color='green', linestyle='-', linewidth='3.5', antialiased=True, alpha = 0.6, label='Lu')
line3, = ax2.plot(dos_H, freq,  color='red', linestyle='-', linewidth='3.5', antialiased=True, alpha = 0.6, label='H')
ax2.fill_between(dos_H, freq ,color='red', alpha=0.15)
ax2.legend(handles=[line1,line2,line3], loc='best', fontsize='12')

## plotting the Eliashberg function
ax3 = ax[2]
ax3.tick_params(axis='both',direction='in',bottom=True, top=True, left=True, right=True)
ax3.set_xlim(0,2)
ax3.set_xticks((np.arange(0,3,step=1)))
ax3.set_xlabel(r'$\alpha^2$F\,($\hbar\omega$)')
ax3.axhline(y=0, color='black', linestyle='--', linewidth=3)
plt.setp(ax3.get_yticklabels(), visible=False)
line1, = ax3.plot(a2f, freq_a2f, color='blue', linestyle='-', linewidth='3.5', antialiased=True, alpha = 0.6, label=r'$\alpha^2$F\,($\omega$)')
ax3.fill_between(a2f, freq_a2f ,color='blue', alpha=0.15)
line2, = ax3.plot(elph, freq_a2f, color='black', linestyle='-', linewidth='3.5', antialiased=True, label='$\lambda(\omega)$')
plt.legend(handles=[line1,line2], loc=3, fontsize='12')

## saving the figures
plt.savefig('phonons.png', bbox_inches='tight', dpi=300)